// @bun
// src/explorer/index.ts
import { EditorView as EditorView3, ViewPlugin as ViewPlugin3 } from "@codemirror/view";

// src/sidebar/index.ts
import { ViewPlugin, EditorView } from "@codemirror/view";
import {
  StateField,
  StateEffect,
  Facet
} from "@codemirror/state";
import crelt from "crelt";

// src/sidebar/styles.ts
var styles = {
  sidebar: "cm-sidebar",
  resizeHandle: "cm-sidebar-resize-handle",
  panelContainer: "cm-sidebar-panel-container"
};
var defaultSidebarOptions = {
  width: "250px",
  backgroundColor: "#21222c",
  overlay: true
};
var inlineStyles = {
  editor: {
    overlay: {
      display: "block",
      position: "relative"
    },
    nonOverlay: {
      display: "flex !important",
      flexDirection: "row !important",
      position: "relative"
    }
  },
  sidebar: {
    base: {
      height: "100%",
      flexShrink: "0",
      display: "flex",
      flexDirection: "column",
      transition: "none",
      minWidth: "0"
    },
    overlay: {
      position: "absolute",
      top: "0",
      zIndex: "10"
    },
    nonOverlay: {
      position: "relative",
      zIndex: "1",
      overflow: "hidden",
      flexShrink: "0",
      flexBasis: "auto"
    }
  },
  resizeHandle: {
    position: "absolute",
    top: "0",
    width: "4px",
    height: "100%",
    cursor: "col-resize",
    zIndex: "20"
  },
  editorContent: {
    flex: "1 1 auto !important",
    width: "auto !important",
    position: "relative",
    minWidth: "0 !important",
    height: "100% !important"
  },
  dragging: {
    cursor: "col-resize",
    userSelect: "none"
  }
};

// src/utils/logger.ts
import pino from "pino";
var isProduction = false;
var logger = pino({
  level: "debug",
  transport: !isProduction ? {
    target: "pino-pretty",
    options: {
      colorize: true,
      translateTime: "SYS:standard"
    }
  } : undefined
});
var logger_default = logger;

// src/sidebar/index.ts
var sidebarPanel = Facet.define({
  combine: (values) => values.flat()
});
var toggleSidebarEffect = StateEffect.define();
var updateSidebarOptionsEffect = StateEffect.define();
var setActivePanelEffect = StateEffect.define();
var sidebarStates = new Map;
var createSidebarState = (id, initialOptions) => {
  return StateField.define({
    create: () => ({
      visible: initialOptions.initiallyOpen ?? false,
      options: initialOptions,
      activePanelId: initialOptions.initiallyOpen && initialOptions.initialPanelId ? initialOptions.initialPanelId : null
    }),
    update(value, tr) {
      let newState = value;
      for (const e of tr.effects) {
        if (e.is(toggleSidebarEffect) && e.value.id === id) {
          newState = { ...newState, visible: e.value.visible };
        } else if (e.is(updateSidebarOptionsEffect) && e.value.id === id) {
          newState = {
            ...newState,
            options: { ...newState.options, ...e.value }
          };
        } else if (e.is(setActivePanelEffect) && e.value.id === id) {
          newState = {
            ...newState,
            activePanelId: e.value.panelId
          };
        }
      }
      return newState;
    }
  });
};
var toggleSidebarCommand = (view, sidebarId) => {
  const stateField = sidebarStates.get(sidebarId);
  if (!stateField) {
    debug("No state field for sidebar:", sidebarId);
    return false;
  }
  const state = view.state.field(stateField);
  debug("Toggling sidebar:", sidebarId, "from:", state.visible, "to:", !state.visible);
  view.dispatch({
    effects: toggleSidebarEffect.of({
      id: sidebarId,
      visible: !state.visible
    })
  });
  return true;
};
var debug = (...args) => logger_default.debug({ module: "Sidebar" }, args.join(" "));
var createSidebarPlugin = (id) => ViewPlugin.fromClass(class {
  dom;
  panelContainer;
  activePanel = null;
  resizeHandle;
  initialWidth = 0;
  initialX = 0;
  isDragging = false;
  constructor(view) {
    this.dom = this.createSidebarDOM();
    this.panelContainer = crelt("div", {
      class: styles.panelContainer
    });
    this.resizeHandle = this.createResizeHandle();
    this.dom.appendChild(this.resizeHandle);
    this.dom.appendChild(this.panelContainer);
    const stateField = sidebarStates.get(id);
    const state = view.state.field(stateField);
    this.applySidebarStyles(state.options, state.visible);
    this.updateVisibility(state.visible);
    view.dom.appendChild(this.dom);
    view.dom.style.position = "relative";
    this.renderActivePanel(view, state);
  }
  update(update) {
    const stateField = sidebarStates.get(id);
    const state = update.state.field(stateField);
    const oldState = update.startState.field(stateField);
    if (state.visible !== oldState.visible) {
      debug("Visibility changed:", id, state.visible);
      this.updateVisibility(state.visible);
      this.applySidebarStyles(state.options, state.visible);
    }
    if (state.options !== oldState.options) {
      debug("Options changed:", id, state.options);
      this.applySidebarStyles(state.options, state.visible);
    }
    if (state.activePanelId !== oldState.activePanelId) {
      debug("Active panel changed:", id, state.activePanelId);
      this.renderActivePanel(update.view, state);
    }
  }
  destroy() {
    debug("Destroying sidebar plugin:", id);
    this.dom.remove();
  }
  createSidebarDOM() {
    return crelt("div", {
      class: styles.sidebar,
      "data-sidebar-id": id
    });
  }
  updateVisibility(visible) {
    this.dom.style.display = visible ? "block" : "none";
  }
  createResizeHandle() {
    const handle = crelt("div", {
      class: styles.resizeHandle
    });
    Object.assign(handle.style, inlineStyles.resizeHandle);
    const startDragging = (e) => {
      e.preventDefault();
      this.isDragging = true;
      this.initialX = e.clientX;
      this.initialWidth = this.dom.offsetWidth;
      document.addEventListener("mousemove", onDrag);
      document.addEventListener("mouseup", stopDragging);
      Object.assign(document.body.style, inlineStyles.dragging);
    };
    const onDrag = (e) => {
      if (!this.isDragging)
        return;
      const stateField = sidebarStates.get(id);
      const editorElement = this.dom.closest(".cm-editor");
      if (!editorElement)
        return;
      const view = EditorView.findFromDOM(editorElement);
      if (!view)
        return;
      const state = view.state.field(stateField);
      const delta = e.clientX - this.initialX;
      let newWidth = state.options.dock === "left" ? this.initialWidth + delta : this.initialWidth - delta;
      newWidth = Math.max(150, Math.min(800, newWidth));
      this.dom.style.width = `${newWidth}px`;
      view.dispatch({
        effects: updateSidebarOptionsEffect.of({
          ...state.options,
          width: `${newWidth}px`
        })
      });
    };
    const stopDragging = () => {
      this.isDragging = false;
      document.removeEventListener("mousemove", onDrag);
      document.removeEventListener("mouseup", stopDragging);
      document.body.style.cursor = "";
      document.body.style.userSelect = "";
    };
    handle.addEventListener("mousedown", startDragging);
    return handle;
  }
  applySidebarStyles(options, visible) {
    const { width, backgroundColor, dock, overlay } = options;
    const editor = this.dom.parentElement;
    if (editor) {
      Object.assign(editor.style, overlay ? inlineStyles.editor.overlay : inlineStyles.editor.nonOverlay);
    }
    Object.assign(this.dom.style, {
      ...inlineStyles.sidebar.base,
      background: backgroundColor
    });
    Object.assign(this.resizeHandle.style, {
      [dock === "left" ? "right" : "left"]: "-2px"
    });
    if (!overlay) {
      Object.assign(this.dom.style, {
        ...inlineStyles.sidebar.nonOverlay,
        order: dock === "left" ? -1 : 1,
        width: visible ? width : "0",
        opacity: visible ? "1" : "0"
      });
    } else {
      Object.assign(this.dom.style, {
        ...inlineStyles.sidebar.overlay,
        [dock === "left" ? "left" : "right"]: visible ? "0" : `-${width}`,
        width
      });
    }
    if (!overlay && editor) {
      const editorContent = editor.querySelector(".cm-scroller");
      if (editorContent) {
        Object.assign(editorContent.style, inlineStyles.editorContent);
      }
    }
  }
  renderActivePanel(view, state) {
    const panelSpecs = view.state.facet(sidebarPanel);
    this.panelContainer.innerHTML = "";
    this.activePanel = null;
    if (state.activePanelId) {
      const panelSpec = panelSpecs.find((spec) => spec.id === state.activePanelId);
      if (panelSpec) {
        debug("Found panel spec:", panelSpec.id);
        this.activePanel = panelSpec.create(view);
        this.panelContainer.appendChild(this.activePanel);
        panelSpec.update?.(view);
      } else {
        debug("Warning: No panel spec found for:", state.activePanelId);
      }
    }
  }
});
function createSidebar(options) {
  const { id } = options;
  const mergedOptions = { ...defaultSidebarOptions, ...options };
  if (!sidebarStates.has(id)) {
    sidebarStates.set(id, createSidebarState(id, mergedOptions));
  }
  const stateField = sidebarStates.get(id);
  return [
    stateField,
    createSidebarPlugin(id),
    EditorView.updateListener.of((update) => {
      const hasToggleEffect = update.transactions.some((tr) => tr.effects.some((e) => e.is(toggleSidebarEffect) && e.value.id === id && e.value.visible));
      if (hasToggleEffect) {
        update.view.dispatch({
          effects: [
            setActivePanelEffect.of({
              id,
              panelId: mergedOptions.initialPanelId ?? null
            })
          ]
        });
      }
    })
  ];
}

// src/sidebar/keymap.ts
import { keymap } from "@codemirror/view";
var debug2 = (...args) => logger_default.debug("[Sidebar Keymap]", ...args);
function createSidebarBindings(sidebarId, key) {
  const bindings = [];
  debug2("Creating keymap bindings for sidebar:", sidebarId, key);
  if (typeof key === "string") {
    debug2("Adding universal key binding:", key);
    bindings.push({
      key,
      run: (view) => {
        debug2("Triggered universal key binding:", key);
        return toggleSidebarCommand(view, sidebarId);
      }
    });
  } else {
    if (key.mac) {
      debug2("Adding Mac key binding:", key.mac);
      bindings.push({
        mac: key.mac,
        run: (view) => {
          debug2("Triggered Mac key binding:", key.mac);
          return toggleSidebarCommand(view, sidebarId);
        }
      });
    }
    if (key.win) {
      debug2("Adding Windows key binding:", key.win);
      bindings.push({
        win: key.win,
        run: (view) => {
          debug2("Triggered Windows key binding:", key.win);
          return toggleSidebarCommand(view, sidebarId);
        }
      });
    }
  }
  return bindings;
}
function createSidebarKeymap(sidebarId, keymapConfig) {
  if (!keymapConfig) {
    debug2("No keymap config provided for sidebar:", sidebarId);
    return [];
  }
  debug2("Creating keymap extension for sidebar:", sidebarId, keymapConfig);
  return keymap.of(createSidebarBindings(sidebarId, keymapConfig));
}

// src/explorer/state.ts
import { StateField as StateField2, StateEffect as StateEffect2, Compartment } from "@codemirror/state";
import { ViewPlugin as ViewPlugin2 } from "@codemirror/view";
import { javascript } from "@codemirror/lang-javascript";
import { python } from "@codemirror/lang-python";
import { markdown } from "@codemirror/lang-markdown";
import { json } from "@codemirror/lang-json";
import crelt2 from "crelt";

// src/explorer/styles.ts
var styles2 = {
  explorerContent: "cm-explorer-content",
  explorerList: "cm-explorer-list",
  explorerItem: "cm-explorer-item",
  explorerDirectory: "cm-explorer-directory",
  explorerDirectoryItem: "cm-explorer-directory-item",
  explorerFile: "cm-explorer-file",
  explorerItemSelected: "cm-explorer-item-selected",
  directoryCaret: "cm-directory-caret",
  directoryCaretExpanded: "cm-directory-caret-expanded"
};
var cssClasses = `
.cm-sidebar-panel-container {
    height: 100%;
    overflow-y: auto;
}

.${styles2.explorerItem} {
    border: 1px solid #2c313a;
    display: flex;
    align-items: center;
    padding: 4px 8px;
    cursor: pointer;
    transition: background-color 0.2s;
}

.${styles2.explorerItem}:hover {
    background-color: rgba(255, 255, 255, 0.05);
}

.${styles2.explorerItemSelected} {
    background-color: rgba(255, 255, 255, 0.05);
    border: 1px solid rgba(255, 255, 255, 0.1);
}
`;
if (typeof document !== "undefined") {
  const style = document.createElement("style");
  style.textContent = cssClasses;
  document.head.appendChild(style);
}
var inlineStyles2 = {
  header: "user-select: none; text-transform: uppercase; font-size: 14px; font-weight: 800; margin: 0; padding: 4px 8px 3px; letter-spacing: 0.5px; color: #e1e1e3;",
  caret: "display: flex; align-items: center; justify-content: center; width: 8px; height: 16px; line-height: 16px; text-align: center; user-select: none; font-size: 13px; opacity: 0.6; transform: rotate(0deg); transition: transform 0.15s ease;",
  caretExpanded: "transform: rotate(90deg)",
  directorySpan: "margin-left: 4px; user-select: none; display: flex; align-items: center; font-size: 14px; font-weight: 500;",
  directoryItem: "padding: 1.5px 0; display: flex; align-items: center;",
  fileSpan: "user-select: none; margin-left: 12px; font-size: 14px; font-weight: 500;",
  fileItem: "padding: 3px 0;",
  list: {
    listStyle: "none",
    padding: "0",
    margin: "0"
  }
};

// src/explorer/state.ts
var debug3 = (...args) => logger_default.debug({ module: "Explorer" }, args.join(" "));
var languageCompartment = new Compartment;
function getLanguageExtension(filename) {
  const ext = filename.toLowerCase().split(".").pop();
  switch (ext) {
    case "js":
    case "jsx":
    case "ts":
    case "tsx":
      return javascript({ typescript: ext.startsWith("ts") });
    case "py":
      return python();
    case "md":
    case "markdown":
      return markdown();
    case "json":
      return json();
    default:
      return null;
  }
}
var selectFileEffect = StateEffect2.define();
var updateFilesEffect = StateEffect2.define();
var toggleDirEffect = StateEffect2.define();
var setProjectNameEffect = StateEffect2.define();
var fileExplorerState = StateField2.define({
  create() {
    return {
      files: [],
      selectedFile: null,
      expandedDirs: new Set,
      projectName: undefined
    };
  },
  update(value, transaction) {
    for (const effect of transaction.effects) {
      if (effect.is(selectFileEffect)) {
        debug3("File selected:", effect.value);
        return { ...value, selectedFile: effect.value };
      } else if (effect.is(updateFilesEffect)) {
        debug3("Files updated:", effect.value.length, "files");
        return {
          ...value,
          files: effect.value,
          selectedFile: null
        };
      } else if (effect.is(toggleDirEffect)) {
        const newExpandedDirs = new Set(value.expandedDirs);
        if (newExpandedDirs.has(effect.value)) {
          newExpandedDirs.delete(effect.value);
        } else {
          newExpandedDirs.add(effect.value);
        }
        debug3("Directory toggled:", effect.value);
        return { ...value, expandedDirs: newExpandedDirs };
      } else if (effect.is(setProjectNameEffect)) {
        debug3("Project name set:", effect.value);
        return { ...value, projectName: effect.value };
      }
    }
    return value;
  }
});
var fileExplorerPanelSpec = {
  id: "file-explorer",
  create(view) {
    debug3("Creating file explorer panel");
    const dom = crelt2("div", { class: styles2.explorerContent });
    renderFileExplorer(dom, view);
    return dom;
  },
  update(view) {
    const dom = view.dom.querySelector(`.${styles2.explorerContent}`);
    if (dom) {
      renderFileExplorer(dom, view);
    }
  }
};
function renderFileExplorer(dom, view) {
  const explorerState = view.state.field(fileExplorerState);
  const header = crelt2("h3", { style: inlineStyles2.header }, explorerState.projectName || "Files");
  const fileList = crelt2("ul", {
    class: styles2.explorerList,
    style: Object.entries(inlineStyles2.list).map(([k, v]) => `${k}:${v}`).join(";")
  });
  const fileTree = buildFileTree(explorerState.files);
  fileTree.forEach((node) => renderFileNode(node, 0, fileList, view, explorerState.selectedFile));
  dom.innerHTML = "";
  dom.appendChild(header);
  dom.appendChild(fileList);
}
function buildFileTree(files) {
  const root = [];
  const directories = {};
  const sortedFiles = [...files].sort((a, b) => a.name.localeCompare(b.name));
  sortedFiles.forEach((file) => {
    const parts = file.name.split("/");
    let currentPath = "";
    let currentChildren = root;
    parts.forEach((part, index) => {
      currentPath = currentPath ? `${currentPath}/${part}` : part;
      const isLastPart = index === parts.length - 1;
      if (isLastPart) {
        currentChildren.push({
          name: part,
          path: currentPath,
          content: file.content,
          isDirectory: false,
          children: []
        });
      } else {
        if (!directories[currentPath]) {
          const dirNode = {
            name: part,
            path: currentPath,
            isDirectory: true,
            children: []
          };
          directories[currentPath] = dirNode;
          currentChildren.push(dirNode);
        }
        currentChildren = directories[currentPath].children;
      }
    });
  });
  return sortNodes(root);
}
function sortNodes(nodes) {
  return nodes.sort((a, b) => {
    if (a.isDirectory !== b.isDirectory) {
      return a.isDirectory ? -1 : 1;
    }
    return a.name.localeCompare(b.name);
  }).map((node) => {
    if (node.isDirectory) {
      node.children = sortNodes(node.children);
    }
    return node;
  });
}
function renderFileNode(node, level, container, view, selectedFile) {
  const indentation = 10 + level * 8;
  const explorerState = view.state.field(fileExplorerState);
  if (node.isDirectory) {
    const isExpanded = explorerState.expandedDirs.has(node.path);
    const caretSpan = crelt2("span", {
      class: `${styles2.directoryCaret}${isExpanded ? ` ${styles2.directoryCaretExpanded}` : ""}`,
      style: `${inlineStyles2.caret}${isExpanded ? `; ${inlineStyles2.caretExpanded}` : ""}`
    }, "\u203A");
    const dirSpan = crelt2("span", {
      class: styles2.explorerDirectory,
      style: inlineStyles2.directorySpan
    }, node.name);
    const dirItem = crelt2("li", {
      class: `${styles2.explorerItem} ${styles2.explorerDirectoryItem}`,
      style: `padding-left: ${indentation}px`,
      onclick: () => {
        view.dispatch({
          effects: toggleDirEffect.of(node.path)
        });
      }
    }, caretSpan, dirSpan);
    container.appendChild(dirItem);
    if (isExpanded) {
      node.children.forEach((child) => renderFileNode(child, level + 1, container, view, selectedFile));
    }
  } else {
    const fileSpan = crelt2("span", {
      class: styles2.explorerFile,
      style: inlineStyles2.fileSpan
    }, node.name);
    const fileItem = crelt2("li", {
      "data-file": node.path,
      class: `${styles2.explorerItem}${node.path === selectedFile ? ` ${styles2.explorerItemSelected}` : ""}`,
      style: `padding-left: ${indentation}px`,
      onclick: () => {
        if (node.content) {
          debug3("Loading file:", node.path);
          const langExtension = getLanguageExtension(node.path);
          view.dispatch({
            effects: [
              selectFileEffect.of(node.path),
              ...langExtension ? [
                languageCompartment.reconfigure(langExtension)
              ] : []
            ],
            changes: {
              from: 0,
              to: view.state.doc.length,
              insert: node.content
            }
          });
        }
      }
    }, fileSpan);
    container.appendChild(fileItem);
  }
}
var fileExplorerPlugin = ViewPlugin2.fromClass(class {
  update(update) {
    if (update.state.field(fileExplorerState) !== update.startState.field(fileExplorerState)) {
      debug3("Explorer state changed, updating panel");
      const dom = update.view.dom.querySelector(`.${styles2.explorerContent}`);
      if (dom) {
        renderFileExplorer(dom, update.view);
      }
    }
  }
});
// src/explorer/commands.ts
function toggleExplorer(view) {
  return toggleSidebarCommand(view, "file-explorer");
}

// src/explorer/index.ts
var globalKeymapHandlers = new Map;
if (typeof window !== "undefined") {
  window.addEventListener("keydown", (event) => {
    const isMac = /Mac/.test(navigator.platform);
    const modKey = isMac ? event.metaKey : event.ctrlKey;
    if (!modKey)
      return;
    const key = `${isMac ? "Cmd" : "Ctrl"}-${event.key.toLowerCase()}`;
    const handler = globalKeymapHandlers.get(key);
    if (handler) {
      const editorElement = document.querySelector(".cm-editor");
      if (editorElement) {
        const view = EditorView3.findFromDOM(editorElement);
        if (view) {
          event.preventDefault();
          handler(view);
        }
      }
    }
  });
}
function createFileSelectPlugin(onFileSelect) {
  return ViewPlugin3.fromClass(class {
    update(update) {
      if (!onFileSelect)
        return;
      for (const effect of update.transactions.flatMap((tr) => tr.effects)) {
        if (effect.is(selectFileEffect)) {
          onFileSelect(effect.value, update.view);
        }
      }
    }
  });
}
function explorer(options = {}) {
  const {
    dock = "left",
    width = "250px",
    overlay = false,
    backgroundColor = "#2c313a",
    keymap: keymap2,
    initiallyOpen = false,
    onFileSelect
  } = options;
  const sidebarOptions = {
    id: "file-explorer",
    dock,
    width,
    overlay,
    backgroundColor,
    initiallyOpen,
    initialPanelId: "file-explorer"
  };
  if (keymap2) {
    if (typeof keymap2 === "string") {
      globalKeymapHandlers.set(keymap2, (view) => toggleSidebarCommand(view, "file-explorer"));
    } else {
      if (keymap2.mac) {
        globalKeymapHandlers.set(keymap2.mac, (view) => toggleSidebarCommand(view, "file-explorer"));
      }
      if (keymap2.win) {
        globalKeymapHandlers.set(keymap2.win, (view) => toggleSidebarCommand(view, "file-explorer"));
      }
    }
  }
  return [
    ...createSidebar(sidebarOptions),
    fileExplorerState,
    fileExplorerPlugin,
    sidebarPanel.of(fileExplorerPanelSpec),
    languageCompartment.of([]),
    onFileSelect ? createFileSelectPlugin(onFileSelect) : [],
    createSidebarKeymap("file-explorer", keymap2)
  ];
}
export {
  updateFilesEffect,
  toggleExplorer,
  setProjectNameEffect,
  selectFileEffect,
  explorer
};
