// @bun
// src/explorer/index.ts
import { EditorView as EditorView3, ViewPlugin as ViewPlugin3 } from "@codemirror/view";

// src/sidebar/index.ts
import { ViewPlugin, EditorView } from "@codemirror/view";
import {
  StateField,
  StateEffect,
  Facet
} from "@codemirror/state";
import crelt from "crelt";

// src/sidebar/styles.ts
var styles = {
  sidebar: "cm-sidebar",
  resizeHandle: "cm-sidebar-resize-handle",
  panelContainer: "cm-sidebar-panel-container"
};
var defaultSidebarOptions = {
  width: "250px",
  backgroundColor: "#21222c",
  overlay: true
};
var inlineStyles = {
  editor: {
    overlay: {
      display: "block",
      position: "relative"
    },
    nonOverlay: {
      display: "flex !important",
      flexDirection: "row !important",
      position: "relative"
    }
  },
  sidebar: {
    base: {
      height: "100%",
      flexShrink: "0",
      display: "flex",
      flexDirection: "column",
      transition: "none",
      minWidth: "0"
    },
    overlay: {
      position: "absolute",
      top: "0",
      zIndex: "10"
    },
    nonOverlay: {
      position: "relative",
      zIndex: "1",
      overflow: "hidden",
      flexShrink: "0",
      flexBasis: "auto"
    }
  },
  resizeHandle: {
    position: "absolute",
    top: "0",
    width: "4px",
    height: "100%",
    cursor: "col-resize",
    zIndex: "20"
  },
  editorContent: {
    flex: "1 1 auto !important",
    width: "auto !important",
    position: "relative",
    minWidth: "0 !important",
    height: "100% !important"
  },
  dragging: {
    cursor: "col-resize",
    userSelect: "none"
  }
};

// src/utils/logger.ts
import pino from "pino";
var isProduction = false;
var logger = pino({
  level: "debug",
  transport: !isProduction ? {
    target: "pino-pretty",
    options: {
      colorize: true,
      translateTime: "SYS:standard"
    }
  } : undefined
});
var logger_default = logger;

// src/sidebar/index.ts
var sidebarPanel = Facet.define({
  combine: (values) => values.flat()
});
var toggleSidebarEffect = StateEffect.define();
var updateSidebarOptionsEffect = StateEffect.define();
var setActivePanelEffect = StateEffect.define();
var sidebarStates = new Map;
var createSidebarState = (id, initialOptions) => {
  return StateField.define({
    create: () => ({
      visible: initialOptions.initiallyOpen ?? false,
      options: initialOptions,
      activePanelId: initialOptions.initiallyOpen && initialOptions.initialPanelId ? initialOptions.initialPanelId : null
    }),
    update(value, tr) {
      let newState = value;
      for (const e of tr.effects) {
        if (e.is(toggleSidebarEffect) && e.value.id === id) {
          newState = { ...newState, visible: e.value.visible };
        } else if (e.is(updateSidebarOptionsEffect) && e.value.id === id) {
          newState = {
            ...newState,
            options: { ...newState.options, ...e.value }
          };
        } else if (e.is(setActivePanelEffect) && e.value.id === id) {
          newState = {
            ...newState,
            activePanelId: e.value.panelId
          };
        }
      }
      return newState;
    }
  });
};
var toggleSidebarCommand = (view, sidebarId) => {
  const stateField = sidebarStates.get(sidebarId);
  if (!stateField) {
    debug("No state field for sidebar:", sidebarId);
    return false;
  }
  const state = view.state.field(stateField);
  debug("Toggling sidebar:", sidebarId, "from:", state.visible, "to:", !state.visible);
  view.dispatch({
    effects: toggleSidebarEffect.of({
      id: sidebarId,
      visible: !state.visible
    })
  });
  return true;
};
var debug = (...args) => logger_default.debug({ module: "Sidebar" }, args.join(" "));
var createSidebarPlugin = (id) => ViewPlugin.fromClass(class {
  dom;
  panelContainer;
  activePanel = null;
  resizeHandle;
  initialWidth = 0;
  initialX = 0;
  isDragging = false;
  constructor(view) {
    this.dom = this.createSidebarDOM();
    this.panelContainer = crelt("div", {
      class: styles.panelContainer
    });
    this.resizeHandle = this.createResizeHandle();
    this.dom.appendChild(this.resizeHandle);
    this.dom.appendChild(this.panelContainer);
    const stateField = sidebarStates.get(id);
    const state = view.state.field(stateField);
    this.applySidebarStyles(state.options, state.visible);
    this.updateVisibility(state.visible);
    view.dom.appendChild(this.dom);
    view.dom.style.position = "relative";
    this.renderActivePanel(view, state);
  }
  update(update) {
    const stateField = sidebarStates.get(id);
    const state = update.state.field(stateField);
    const oldState = update.startState.field(stateField);
    if (state.visible !== oldState.visible) {
      debug("Visibility changed:", id, state.visible);
      this.updateVisibility(state.visible);
      this.applySidebarStyles(state.options, state.visible);
    }
    if (state.options !== oldState.options) {
      debug("Options changed:", id, state.options);
      this.applySidebarStyles(state.options, state.visible);
    }
    if (state.activePanelId !== oldState.activePanelId) {
      debug("Active panel changed:", id, state.activePanelId);
      this.renderActivePanel(update.view, state);
    }
  }
  destroy() {
    debug("Destroying sidebar plugin:", id);
    this.dom.remove();
  }
  createSidebarDOM() {
    return crelt("div", {
      class: styles.sidebar,
      "data-sidebar-id": id
    });
  }
  updateVisibility(visible) {
    this.dom.style.display = visible ? "block" : "none";
  }
  createResizeHandle() {
    const handle = crelt("div", {
      class: styles.resizeHandle
    });
    Object.assign(handle.style, inlineStyles.resizeHandle);
    const startDragging = (e) => {
      e.preventDefault();
      this.isDragging = true;
      this.initialX = e.clientX;
      this.initialWidth = this.dom.offsetWidth;
      document.addEventListener("mousemove", onDrag);
      document.addEventListener("mouseup", stopDragging);
      Object.assign(document.body.style, inlineStyles.dragging);
    };
    const onDrag = (e) => {
      if (!this.isDragging)
        return;
      const stateField = sidebarStates.get(id);
      const editorElement = this.dom.closest(".cm-editor");
      if (!editorElement)
        return;
      const view = EditorView.findFromDOM(editorElement);
      if (!view)
        return;
      const state = view.state.field(stateField);
      const delta = e.clientX - this.initialX;
      let newWidth = state.options.dock === "left" ? this.initialWidth + delta : this.initialWidth - delta;
      newWidth = Math.max(150, Math.min(800, newWidth));
      this.dom.style.width = `${newWidth}px`;
      view.dispatch({
        effects: updateSidebarOptionsEffect.of({
          ...state.options,
          width: `${newWidth}px`
        })
      });
    };
    const stopDragging = () => {
      this.isDragging = false;
      document.removeEventListener("mousemove", onDrag);
      document.removeEventListener("mouseup", stopDragging);
      document.body.style.cursor = "";
      document.body.style.userSelect = "";
    };
    handle.addEventListener("mousedown", startDragging);
    return handle;
  }
  applySidebarStyles(options, visible) {
    const { width, backgroundColor, dock, overlay } = options;
    const editor = this.dom.parentElement;
    if (editor) {
      Object.assign(editor.style, overlay ? inlineStyles.editor.overlay : inlineStyles.editor.nonOverlay);
    }
    Object.assign(this.dom.style, {
      ...inlineStyles.sidebar.base,
      background: backgroundColor
    });
    Object.assign(this.resizeHandle.style, {
      [dock === "left" ? "right" : "left"]: "-2px"
    });
    if (!overlay) {
      Object.assign(this.dom.style, {
        ...inlineStyles.sidebar.nonOverlay,
        order: dock === "left" ? -1 : 1,
        width: visible ? width : "0",
        opacity: visible ? "1" : "0"
      });
    } else {
      Object.assign(this.dom.style, {
        ...inlineStyles.sidebar.overlay,
        [dock === "left" ? "left" : "right"]: visible ? "0" : `-${width}`,
        width
      });
    }
    if (!overlay && editor) {
      const editorContent = editor.querySelector(".cm-scroller");
      if (editorContent) {
        Object.assign(editorContent.style, inlineStyles.editorContent);
      }
    }
  }
  renderActivePanel(view, state) {
    const panelSpecs = view.state.facet(sidebarPanel);
    this.panelContainer.innerHTML = "";
    this.activePanel = null;
    if (state.activePanelId) {
      const panelSpec = panelSpecs.find((spec) => spec.id === state.activePanelId);
      if (panelSpec) {
        debug("Found panel spec:", panelSpec.id);
        this.activePanel = panelSpec.create(view);
        this.panelContainer.appendChild(this.activePanel);
        panelSpec.update?.(view);
      } else {
        debug("Warning: No panel spec found for:", state.activePanelId);
      }
    }
  }
});
function createSidebar(options) {
  const { id } = options;
  const mergedOptions = { ...defaultSidebarOptions, ...options };
  if (!sidebarStates.has(id)) {
    sidebarStates.set(id, createSidebarState(id, mergedOptions));
  }
  const stateField = sidebarStates.get(id);
  return [
    stateField,
    createSidebarPlugin(id),
    EditorView.updateListener.of((update) => {
      const hasToggleEffect = update.transactions.some((tr) => tr.effects.some((e) => e.is(toggleSidebarEffect) && e.value.id === id && e.value.visible));
      if (hasToggleEffect) {
        update.view.dispatch({
          effects: [
            setActivePanelEffect.of({
              id,
              panelId: mergedOptions.initialPanelId ?? null
            })
          ]
        });
      }
    })
  ];
}

// src/sidebar/keymap.ts
import { keymap } from "@codemirror/view";
var debug2 = (...args) => logger_default.debug("[Sidebar Keymap]", ...args);
function createSidebarBindings(sidebarId, key) {
  const bindings = [];
  debug2("Creating keymap bindings for sidebar:", sidebarId, key);
  if (typeof key === "string") {
    debug2("Adding universal key binding:", key);
    bindings.push({
      key,
      run: (view) => {
        debug2("Triggered universal key binding:", key);
        return toggleSidebarCommand(view, sidebarId);
      }
    });
  } else {
    if (key.mac) {
      debug2("Adding Mac key binding:", key.mac);
      bindings.push({
        mac: key.mac,
        run: (view) => {
          debug2("Triggered Mac key binding:", key.mac);
          return toggleSidebarCommand(view, sidebarId);
        }
      });
    }
    if (key.win) {
      debug2("Adding Windows key binding:", key.win);
      bindings.push({
        win: key.win,
        run: (view) => {
          debug2("Triggered Windows key binding:", key.win);
          return toggleSidebarCommand(view, sidebarId);
        }
      });
    }
  }
  return bindings;
}
function createSidebarKeymap(sidebarId, keymapConfig) {
  if (!keymapConfig) {
    debug2("No keymap config provided for sidebar:", sidebarId);
    return [];
  }
  debug2("Creating keymap extension for sidebar:", sidebarId, keymapConfig);
  return keymap.of(createSidebarBindings(sidebarId, keymapConfig));
}

// src/explorer/state.ts
import { StateField as StateField2, StateEffect as StateEffect2, Compartment } from "@codemirror/state";
import { ViewPlugin as ViewPlugin2 } from "@codemirror/view";
import { javascript } from "@codemirror/lang-javascript";
import { python } from "@codemirror/lang-python";
import { markdown } from "@codemirror/lang-markdown";
import { json } from "@codemirror/lang-json";
import crelt2 from "crelt";

// src/explorer/styles.ts
var styles2 = {
  explorerContent: "cm-explorer-content",
  explorerList: "cm-explorer-list",
  explorerItem: "cm-explorer-item",
  explorerDirectory: "cm-explorer-directory",
  explorerDirectoryItem: "cm-explorer-directory-item",
  explorerFile: "cm-explorer-file",
  explorerItemSelected: "cm-explorer-item-selected",
  directoryCaret: "cm-directory-caret",
  directoryCaretExpanded: "cm-directory-caret-expanded"
};
var cssClasses = `
.cm-sidebar-panel-container {
    height: 100%;
    overflow-y: auto;
}

.${styles2.explorerItem} {
    border: 1px solid #2c313a;
    display: flex;
    align-items: center;
    padding: 4px 8px;
    cursor: pointer;
    transition: background-color 0.2s;
}

.${styles2.explorerItem}:hover {
    background-color: rgba(255, 255, 255, 0.05);
}

.${styles2.explorerItemSelected} {
    background-color: rgba(255, 255, 255, 0.05);
    border: 1px solid rgba(255, 255, 255, 0.1);
}
`;
if (typeof document !== "undefined") {
  const style = document.createElement("style");
  style.textContent = cssClasses;
  document.head.appendChild(style);
}
var inlineStyles2 = {
  header: "user-select: none; text-transform: uppercase; font-size: 14px; font-weight: 800; margin: 0; padding: 4px 8px 3px; letter-spacing: 0.5px; color: #e1e1e3;",
  caret: "display: flex; align-items: center; justify-content: center; width: 8px; height: 16px; line-height: 16px; text-align: center; user-select: none; font-size: 13px; opacity: 0.6; transform: rotate(0deg); transition: transform 0.15s ease;",
  caretExpanded: "transform: rotate(90deg)",
  directorySpan: "margin-left: 4px; user-select: none; display: flex; align-items: center; font-size: 14px; font-weight: 500;",
  directoryItem: "padding: 1.5px 0; display: flex; align-items: center;",
  fileSpan: "user-select: none; margin-left: 12px; font-size: 14px; font-weight: 500;",
  fileItem: "padding: 3px 0;",
  list: {
    listStyle: "none",
    padding: "0",
    margin: "0"
  }
};

// src/explorer/state.ts
var debug3 = (...args) => logger_default.debug({ module: "Explorer" }, args.join(" "));
var languageCompartment = new Compartment;
function getLanguageExtension(filename) {
  const ext = filename.toLowerCase().split(".").pop();
  switch (ext) {
    case "js":
    case "jsx":
    case "ts":
    case "tsx":
      return javascript({ typescript: ext.startsWith("ts") });
    case "py":
      return python();
    case "md":
    case "markdown":
      return markdown();
    case "json":
      return json();
    default:
      return null;
  }
}
var selectFileEffect = StateEffect2.define();
var updateFilesEffect = StateEffect2.define();
var toggleDirEffect = StateEffect2.define();
var setProjectNameEffect = StateEffect2.define();
var fileExplorerState = StateField2.define({
  create() {
    return {
      files: [],
      selectedFile: null,
      expandedDirs: new Set,
      projectName: undefined
    };
  },
  update(value, transaction) {
    for (const effect of transaction.effects) {
      if (effect.is(selectFileEffect)) {
        debug3("File selected:", effect.value);
        return { ...value, selectedFile: effect.value };
      } else if (effect.is(updateFilesEffect)) {
        debug3("Files updated:", effect.value.length, "files");
        return {
          ...value,
          files: effect.value,
          selectedFile: null
        };
      } else if (effect.is(toggleDirEffect)) {
        const newExpandedDirs = new Set(value.expandedDirs);
        if (newExpandedDirs.has(effect.value)) {
          newExpandedDirs.delete(effect.value);
        } else {
          newExpandedDirs.add(effect.value);
        }
        debug3("Directory toggled:", effect.value);
        return { ...value, expandedDirs: newExpandedDirs };
      } else if (effect.is(setProjectNameEffect)) {
        debug3("Project name set:", effect.value);
        return { ...value, projectName: effect.value };
      }
    }
    return value;
  }
});
var fileExplorerPanelSpec = {
  id: "file-explorer",
  create(view) {
    debug3("Creating file explorer panel");
    const dom = crelt2("div", { class: styles2.explorerContent });
    renderFileExplorer(dom, view);
    return dom;
  },
  update(view) {
    const dom = view.dom.querySelector(`.${styles2.explorerContent}`);
    if (dom) {
      renderFileExplorer(dom, view);
    }
  }
};
function renderFileExplorer(dom, view) {
  const explorerState = view.state.field(fileExplorerState);
  const header = crelt2("h3", { style: inlineStyles2.header }, explorerState.projectName || "Files");
  const fileList = crelt2("ul", {
    class: styles2.explorerList,
    style: Object.entries(inlineStyles2.list).map(([k, v]) => `${k}:${v}`).join(";")
  });
  const fileTree = buildFileTree(explorerState.files);
  fileTree.forEach((node) => renderFileNode(node, 0, fileList, view, explorerState.selectedFile));
  dom.innerHTML = "";
  dom.appendChild(header);
  dom.appendChild(fileList);
}
function buildFileTree(files) {
  const root = [];
  const directories = {};
  const sortedFiles = [...files].sort((a, b) => a.name.localeCompare(b.name));
  sortedFiles.forEach((file) => {
    const parts = file.name.split("/");
    let currentPath = "";
    let currentChildren = root;
    parts.forEach((part, index) => {
      currentPath = currentPath ? `${currentPath}/${part}` : part;
      const isLastPart = index === parts.length - 1;
      if (isLastPart) {
        currentChildren.push({
          name: part,
          path: currentPath,
          content: file.content,
          isDirectory: false,
          children: []
        });
      } else {
        if (!directories[currentPath]) {
          const dirNode = {
            name: part,
            path: currentPath,
            isDirectory: true,
            children: []
          };
          directories[currentPath] = dirNode;
          currentChildren.push(dirNode);
        }
        currentChildren = directories[currentPath].children;
      }
    });
  });
  return sortNodes(root);
}
function sortNodes(nodes) {
  return nodes.sort((a, b) => {
    if (a.isDirectory !== b.isDirectory) {
      return a.isDirectory ? -1 : 1;
    }
    return a.name.localeCompare(b.name);
  }).map((node) => {
    if (node.isDirectory) {
      node.children = sortNodes(node.children);
    }
    return node;
  });
}
function renderFileNode(node, level, container, view, selectedFile) {
  const indentation = 10 + level * 8;
  const explorerState = view.state.field(fileExplorerState);
  if (node.isDirectory) {
    const isExpanded = explorerState.expandedDirs.has(node.path);
    const caretSpan = crelt2("span", {
      class: `${styles2.directoryCaret}${isExpanded ? ` ${styles2.directoryCaretExpanded}` : ""}`,
      style: `${inlineStyles2.caret}${isExpanded ? `; ${inlineStyles2.caretExpanded}` : ""}`
    }, "\u203A");
    const dirSpan = crelt2("span", {
      class: styles2.explorerDirectory,
      style: inlineStyles2.directorySpan
    }, node.name);
    const dirItem = crelt2("li", {
      class: `${styles2.explorerItem} ${styles2.explorerDirectoryItem}`,
      style: `padding-left: ${indentation}px`,
      onclick: () => {
        view.dispatch({
          effects: toggleDirEffect.of(node.path)
        });
      }
    }, caretSpan, dirSpan);
    container.appendChild(dirItem);
    if (isExpanded) {
      node.children.forEach((child) => renderFileNode(child, level + 1, container, view, selectedFile));
    }
  } else {
    const fileSpan = crelt2("span", {
      class: styles2.explorerFile,
      style: inlineStyles2.fileSpan
    }, node.name);
    const fileItem = crelt2("li", {
      "data-file": node.path,
      class: `${styles2.explorerItem}${node.path === selectedFile ? ` ${styles2.explorerItemSelected}` : ""}`,
      style: `padding-left: ${indentation}px`,
      onclick: () => {
        if (node.content) {
          debug3("Loading file:", node.path);
          const langExtension = getLanguageExtension(node.path);
          view.dispatch({
            effects: [
              selectFileEffect.of(node.path),
              ...langExtension ? [
                languageCompartment.reconfigure(langExtension)
              ] : []
            ],
            changes: {
              from: 0,
              to: view.state.doc.length,
              insert: node.content
            }
          });
        }
      }
    }, fileSpan);
    container.appendChild(fileItem);
  }
}
var fileExplorerPlugin = ViewPlugin2.fromClass(class {
  update(update) {
    if (update.state.field(fileExplorerState) !== update.startState.field(fileExplorerState)) {
      debug3("Explorer state changed, updating panel");
      const dom = update.view.dom.querySelector(`.${styles2.explorerContent}`);
      if (dom) {
        renderFileExplorer(dom, update.view);
      }
    }
  }
});
// src/explorer/commands.ts
function toggleExplorer(view) {
  return toggleSidebarCommand(view, "file-explorer");
}

// src/explorer/index.ts
var globalKeymapHandlers = new Map;
if (typeof window !== "undefined") {
  window.addEventListener("keydown", (event) => {
    const isMac = /Mac/.test(navigator.platform);
    const modKey = isMac ? event.metaKey : event.ctrlKey;
    if (!modKey)
      return;
    const key = `${isMac ? "Cmd" : "Ctrl"}-${event.key.toLowerCase()}`;
    const handler = globalKeymapHandlers.get(key);
    if (handler) {
      const editorElement = document.querySelector(".cm-editor");
      if (editorElement) {
        const view = EditorView3.findFromDOM(editorElement);
        if (view) {
          event.preventDefault();
          handler(view);
        }
      }
    }
  });
}
function createFileSelectPlugin(onFileSelect) {
  return ViewPlugin3.fromClass(class {
    update(update) {
      if (!onFileSelect)
        return;
      for (const effect of update.transactions.flatMap((tr) => tr.effects)) {
        if (effect.is(selectFileEffect)) {
          onFileSelect(effect.value, update.view);
        }
      }
    }
  });
}
function explorer(options = {}) {
  const {
    dock = "left",
    width = "250px",
    overlay = false,
    backgroundColor = "#2c313a",
    keymap: keymap2,
    initiallyOpen = false,
    onFileSelect
  } = options;
  const sidebarOptions = {
    id: "file-explorer",
    dock,
    width,
    overlay,
    backgroundColor,
    initiallyOpen,
    initialPanelId: "file-explorer"
  };
  if (keymap2) {
    if (typeof keymap2 === "string") {
      globalKeymapHandlers.set(keymap2, (view) => toggleSidebarCommand(view, "file-explorer"));
    } else {
      if (keymap2.mac) {
        globalKeymapHandlers.set(keymap2.mac, (view) => toggleSidebarCommand(view, "file-explorer"));
      }
      if (keymap2.win) {
        globalKeymapHandlers.set(keymap2.win, (view) => toggleSidebarCommand(view, "file-explorer"));
      }
    }
  }
  return [
    ...createSidebar(sidebarOptions),
    fileExplorerState,
    fileExplorerPlugin,
    sidebarPanel.of(fileExplorerPanelSpec),
    languageCompartment.of([]),
    onFileSelect ? createFileSelectPlugin(onFileSelect) : [],
    createSidebarKeymap("file-explorer", keymap2)
  ];
}

// src/assistant/index.ts
import { ViewPlugin as ViewPlugin4 } from "@codemirror/view";

// src/assistant/lib/state.ts
import { StateEffect as StateEffect3, StateField as StateField3 } from "@codemirror/state";
var switchTabEffect = StateEffect3.define();
var addMessageEffect = StateEffect3.define();
var updateMessageStatusEffect = StateEffect3.define();
var selectModelEffect = StateEffect3.define();
var setApiKeyEffect = StateEffect3.define();
var toggleSettingsEffect = StateEffect3.define();
var assistantState = StateField3.define({
  create() {
    const defaultModelId = "gpt-4";
    const savedApiKeys = localStorage.getItem("cm-ai-api-keys");
    const apiKeys = savedApiKeys ? JSON.parse(savedApiKeys) : {
      google: "",
      openai: "",
      mistral: ""
    };
    return {
      activeTab: "assistant",
      messages: [],
      isLoading: false,
      selectedModel: defaultModelId,
      apiKeys,
      showSettings: false
    };
  },
  update(value, tr) {
    for (const effect of tr.effects) {
      if (effect.is(switchTabEffect)) {
        return { ...value, activeTab: effect.value };
      } else if (effect.is(addMessageEffect)) {
        return {
          ...value,
          messages: [...value.messages, effect.value],
          isLoading: effect.value.status === "sending" || effect.value.status === "streaming"
        };
      } else if (effect.is(updateMessageStatusEffect)) {
        return {
          ...value,
          messages: value.messages.map((msg) => msg.id === effect.value.message.id ? {
            ...msg,
            status: effect.value.status,
            content: effect.value.content ?? msg.content
          } : msg),
          isLoading: effect.value.status === "sending" || effect.value.status === "streaming"
        };
      } else if (effect.is(selectModelEffect)) {
        return { ...value, selectedModel: effect.value };
      } else if (effect.is(setApiKeyEffect)) {
        const newState = {
          ...value,
          apiKeys: {
            ...value.apiKeys,
            [effect.value.provider]: effect.value.key
          }
        };
        localStorage.setItem("cm-ai-api-keys", JSON.stringify(newState.apiKeys));
        return newState;
      } else if (effect.is(toggleSettingsEffect)) {
        return { ...value, showSettings: effect.value };
      }
    }
    return value;
  }
});

// src/assistant/lib/renderer.ts
import crelt3 from "crelt";

// src/assistant/lib/codeblock.ts
import { StateField as StateField4, StateEffect as StateEffect4, EditorState } from "@codemirror/state";
import { EditorView as EditorView5, Decoration, WidgetType } from "@codemirror/view";
import { javascript as javascript2 } from "@codemirror/lang-javascript";
import { python as python2 } from "@codemirror/lang-python";
import { markdown as markdown2 } from "@codemirror/lang-markdown";
import { oneDark } from "@codemirror/theme-one-dark";

// src/assistant/lib/styles.ts
import { EditorView as EditorView4 } from "@codemirror/view";
var containerStyles = {
  display: "flex",
  flexDirection: "column",
  height: "100%",
  overflow: "hidden",
  boxSizing: "border-box",
  padding: "12px 12px 6px 12px"
};
var tabsContainerStyles = {
  display: "flex",
  justifyContent: "space-between",
  alignItems: "center",
  height: "37px"
};
var tabsGroupStyles = {
  display: "flex",
  alignItems: "center",
  gap: "4px"
};
var getTabStyles = (isActive) => ({
  background: isActive ? "var(--cm-selected-bg, rgba(255, 255, 255, 0.05))" : "none",
  border: "none",
  padding: "6px 10px",
  fontSize: "13px",
  fontWeight: "500",
  cursor: "pointer",
  borderRadius: "4px 4px 0 0",
  color: "var(--cm-text-color, #e1e1e3)",
  opacity: isActive ? "1" : "0.7",
  transition: "opacity 0.2s",
  marginBottom: "-1px",
  borderBottom: isActive ? "1px solid var(--cm-selected-bg, rgba(255, 255, 255, 0.05))" : "none"
});
var controlsContainerStyles = {
  display: "flex",
  alignItems: "center",
  gap: "8px"
};
var modelSelectStyles = {
  background: "transparent",
  border: "transparent",
  borderRadius: "4px",
  color: "var(--cm-text-color, #e1e1e3)",
  padding: "4px 24px 4px 8px",
  fontSize: "13px",
  cursor: "pointer",
  outline: "none",
  appearance: "none",
  backgroundImage: 'url("data:image/svg+xml;charset=US-ASCII,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2214%22%20height%3D%2214%22%20viewBox%3D%220%200%2014%2014%22%3E%3Cpath%20fill%3D%22%23e1e1e3%22%20d%3D%22M7%2010L3.5%206h7L7%2010z%22%2F%3E%3C%2Fsvg%3E")',
  backgroundRepeat: "no-repeat",
  backgroundPosition: "right 4px center",
  textAlign: "right",
  direction: "rtl"
};
var settingsButtonStyles = {
  background: "none",
  border: "none",
  padding: "6px 8px",
  fontSize: "13px",
  cursor: "pointer",
  opacity: "0.7",
  transition: "opacity 0.2s",
  color: "var(--cm-text-color, #e1e1e3)",
  display: "flex",
  alignItems: "center"
};
var headerBorderStyles = {
  height: "1px",
  background: "var(--cm-border-color, rgba(255, 255, 255, 0.1))",
  margin: "-2px 0 12px"
};
var messagesContainerStyles = {
  flex: "1",
  overflowY: "auto",
  padding: "8px 0",
  display: "flex",
  flexDirection: "column",
  gap: "8px"
};
var inputContainerStyles = {
  borderTop: "1px solid var(--cm-border-color, rgba(255, 255, 255, 0.1))",
  padding: "12px 12px 6px 12px",
  width: "100%",
  boxSizing: "border-box"
};
var textareaStyles = {
  width: "100%",
  boxSizing: "border-box",
  background: "var(--cm-input-bg, rgba(255, 255, 255, 0.05))",
  border: "1px solid var(--cm-border-color, rgba(255, 255, 255, 0.1))",
  borderRadius: "4px",
  color: "var(--cm-text-color, #cdc8d0)",
  padding: "8px 12px",
  fontSize: "13px",
  lineHeight: "1.4",
  resize: "none",
  outline: "none"
};
var loadingContainerStyles = {
  display: "flex",
  alignItems: "center",
  justifyContent: "center",
  padding: "4px 0",
  alignSelf: "flex-start",
  opacity: "0.7"
};
var spinnerStyles = {
  width: "12px",
  height: "12px",
  border: "1.5px solid var(--cm-border-color, rgba(255, 255, 255, 0.05))",
  borderTop: "1.5px solid var(--cm-text-color, #cdc8d0)",
  borderRadius: "50%",
  animation: "cm-spin 0.8s linear infinite"
};
var getMessageStyles = (isUser) => ({
  maxWidth: "85%",
  padding: "8px 10px",
  borderRadius: "12px",
  fontSize: "13px",
  lineHeight: "1.4",
  alignSelf: isUser ? "flex-end" : "flex-start",
  background: isUser ? "var(--cm-accent-color, rgba(74, 158, 255, 0.2))" : "var(--cm-message-bg, rgba(255, 255, 255, 0.05))",
  color: isUser ? "var(--cm-message-text, white)" : "var(--cm-text-color, #cdc8d0)",
  display: "flex",
  flexDirection: "column",
  gap: "4px"
});
var messageContentStyles = {
  whiteSpace: "pre-wrap",
  lineHeight: "1.2",
  display: "flex",
  flexDirection: "column",
  gap: "0"
};
var codeBlockContainerStyles = {
  margin: "0",
  position: "relative",
  border: "1px solid var(--cm-border-color, rgba(255, 255, 255, 0.1))",
  borderRadius: "8px",
  overflow: "hidden",
  backgroundColor: "#1e1e1e",
  boxShadow: "0 2px 4px rgba(0, 0, 0, 0.05)"
};
var codeBlockEditorTheme = EditorView4.theme({
  "&": {
    backgroundColor: "transparent !important",
    height: "auto !important",
    flex: "initial !important",
    position: "static !important"
  },
  ".cm-content": {
    padding: "16px !important",
    height: "auto !important",
    fontFamily: "var(--cm-font-family-mono, ui-monospace, SFMono-Regular, SF Mono, Menlo, Consolas, Liberation Mono, monospace) !important",
    fontSize: "13px !important",
    lineHeight: "1.5 !important"
  },
  ".cm-line": {
    padding: "0 !important"
  },
  ".cm-scroller": {
    height: "auto !important",
    flex: "initial !important",
    width: "100% !important"
  }
});
var codeBlockHeaderStyles = {
  padding: "8px 12px",
  borderBottom: "1px solid rgba(255, 255, 255, 0.1)",
  background: "linear-gradient(to bottom, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0.05))",
  color: "var(--cm-text-secondary, rgba(255, 255, 255, 0.6))",
  fontSize: "11px",
  fontFamily: "var(--cm-font-family-mono, ui-monospace, SFMono-Regular, SF Mono, Menlo, Consolas, Liberation Mono, monospace)",
  textTransform: "lowercase",
  letterSpacing: "0.5px",
  display: "flex",
  alignItems: "center",
  gap: "6px",
  userSelect: "none",
  position: "relative",
  height: "37px",
  boxSizing: "border-box"
};
var copyButtonStyles = {
  background: "none",
  border: "none",
  padding: "4px",
  cursor: "pointer",
  color: "inherit",
  opacity: "0.5",
  transition: "opacity 0.2s",
  marginLeft: "auto",
  display: "flex",
  alignItems: "center",
  justifyContent: "center",
  height: "16px"
};
var dotStyles = {
  fontSize: "8px",
  opacity: "0.5"
};
var textContainerStyles = {
  margin: "0",
  padding: "0",
  lineHeight: "inherit",
  minHeight: "20px"
};
var settingsHeaderStyles = {
  fontSize: "13px",
  fontWeight: "500",
  color: "var(--cm-text-color, #e1e1e3)",
  padding: "6px 10px",
  background: "var(--cm-selected-bg, rgba(255, 255, 255, 0.05))",
  borderRadius: "4px 4px 0 0",
  marginBottom: "-1px",
  borderBottom: "1px solid var(--cm-selected-bg, rgba(255, 255, 255, 0.05))",
  display: "inline-block"
};
var providerHeaderStyles = {
  fontSize: "13px",
  fontWeight: "600",
  color: "var(--cm-text-color, #e1e1e3)",
  padding: "0 0 8px 0",
  display: "flex",
  justifyContent: "space-between",
  alignItems: "center",
  width: "100%",
  boxSizing: "border-box",
  height: "21px"
};
var providerContentStyles = {
  display: "flex",
  flexDirection: "column",
  gap: "2px",
  padding: "2px",
  background: "var(--cm-input-bg, rgba(255, 255, 255, 0.03))",
  borderRadius: "4px"
};
var settingsDescriptionStyles = {
  fontSize: "12px",
  marginTop: "12px",
  marginBottom: "16px",
  color: "var(--cm-text-color, #e1e1e3)",
  opacity: "0.7"
};
var settingsInputsContainerStyles = {
  display: "flex",
  flexDirection: "column",
  gap: "24px"
};
var getSettingsInputStyles = (hasApiKey) => ({
  display: hasApiKey ? "none" : "flex",
  gap: "8px",
  padding: "8px",
  borderRadius: "6px",
  height: "49px",
  boxSizing: "border-box"
});
var settingsInputFieldStyles = {
  flex: "1",
  background: "var(--cm-input-bg, rgba(0, 0, 0, 0.2))",
  border: "1px solid var(--cm-border-color, rgba(255, 255, 255, 0.1))",
  borderRadius: "4px",
  padding: "8px 12px",
  fontSize: "13px",
  color: "var(--cm-text-color, #e1e1e3)",
  outline: "none",
  transition: "all 0.2s ease",
  fontFamily: "monospace",
  boxSizing: "border-box"
};
var settingsMessageContainerStyles = (hasApiKey) => ({
  fontSize: "13px",
  color: "var(--cm-text-color, #e1e1e3)",
  opacity: "0.8",
  padding: "8px 12px",
  display: hasApiKey ? "flex" : "none",
  alignItems: "center",
  background: "var(--cm-input-bg, rgba(0, 0, 0, 0.2))",
  borderRadius: "4px",
  height: "49px",
  boxSizing: "border-box"
});
var inlineCodeStyles = `
    background: var(--cm-code-bg, rgba(96, 125, 139, 0.3));
    padding: 1px 4px;
    border-radius: 3px;
    font-family: monospace;
    font-size: 12px;
    display: inline;
    white-space: pre;
    color: var(--cm-code-color, #e5e7eb);
`;
var codeBlockWrapperStyles = {
  border: "1px solid var(--cm-border-color, rgba(255, 255, 255, 0.1))",
  borderRadius: "8px",
  margin: "0",
  overflow: "hidden",
  backgroundColor: "#1e1e1e",
  boxShadow: "0 2px 4px rgba(0, 0, 0, 0.1)"
};
var codeBlockContentStyles = {
  padding: "16px"
};
var loadingDotsStyles = {
  display: "inline-flex",
  alignItems: "center",
  gap: "3px",
  marginLeft: "auto",
  opacity: "0.5",
  height: "16px"
};
var loadingDotBaseStyles = {
  width: "3px",
  height: "3px",
  borderRadius: "50%",
  transition: "background-color 0.15s ease"
};
var loadingDotFilledStyles = {
  ...loadingDotBaseStyles,
  backgroundColor: "currentColor"
};
var loadingDotEmptyStyles = {
  ...loadingDotBaseStyles,
  backgroundColor: "rgba(255, 255, 255, 0.1)"
};
var settingsPanelContainerStyles = {
  display: "flex",
  flexDirection: "column",
  height: "100%",
  overflow: "hidden",
  boxSizing: "border-box",
  padding: "12px 12px 6px 12px"
};

// src/assistant/lib/codeblock.ts
var getLanguageSupport = (lang) => {
  const normalizedLang = lang.toLowerCase().trim();
  switch (normalizedLang) {
    case "javascript":
    case "js":
      return javascript2();
    case "typescript":
    case "ts":
      return javascript2({ typescript: true });
    case "python":
    case "py":
      return python2();
    case "markdown":
    case "md":
      return markdown2();
    default:
      return null;
  }
};

class CodeBlockWidget extends WidgetType {
  code;
  language;
  isIncomplete;
  view = null;
  static chunkCounter = 0;
  constructor(code, language, isIncomplete = false) {
    super();
    this.code = code;
    this.language = language;
    this.isIncomplete = isIncomplete;
  }
  eq(other) {
    return this.code === other.code && this.language === other.language && this.isIncomplete === other.isIncomplete;
  }
  destroy() {
    this.view?.destroy();
  }
  toDOM() {
    const wrapper = document.createElement("div");
    Object.assign(wrapper.style, codeBlockWrapperStyles);
    wrapper.className = "cm-ai-codeblock";
    if (this.language) {
      const header = document.createElement("div");
      Object.assign(header.style, codeBlockHeaderStyles);
      header.className = "cm-ai-codeblock-header";
      const dot = document.createElement("span");
      dot.textContent = "\u25CF";
      Object.assign(dot.style, dotStyles);
      header.appendChild(dot);
      const langText = document.createElement("span");
      langText.textContent = this.language;
      header.appendChild(langText);
      if (this.isIncomplete) {
        CodeBlockWidget.chunkCounter = (CodeBlockWidget.chunkCounter + 1) % 3;
        const loadingDots = document.createElement("div");
        Object.assign(loadingDots.style, loadingDotsStyles);
        for (let i = 0;i < 3; i++) {
          const dot2 = document.createElement("div");
          Object.assign(dot2.style, i <= CodeBlockWidget.chunkCounter ? loadingDotFilledStyles : loadingDotEmptyStyles);
          loadingDots.appendChild(dot2);
        }
        header.appendChild(loadingDots);
      }
      if (!this.isIncomplete) {
        const copyButton = document.createElement("button");
        Object.assign(copyButton.style, copyButtonStyles);
        copyButton.title = "Copy code";
        copyButton.innerHTML = `<svg width="14" height="14" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round">
                    <rect x="9" y="9" width="13" height="13" rx="2" ry="2"></rect>
                    <path d="M5 15H4a2 2 0 0 1-2-2V4a2 2 0 0 1 2-2h9a2 2 0 0 1 2 2v1"></path>
                </svg>`;
        copyButton.addEventListener("click", async () => {
          try {
            await navigator.clipboard.writeText(this.code);
            const originalHTML = copyButton.innerHTML;
            copyButton.innerHTML = `<svg width="14" height="14" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round">
                            <path d="M20 6L9 17l-5-5"></path>
                        </svg>`;
            setTimeout(() => {
              copyButton.innerHTML = originalHTML;
            }, 2000);
          } catch (err) {
            console.error("Failed to copy code:", err);
          }
        });
        header.appendChild(copyButton);
      }
      wrapper.appendChild(header);
    }
    if (!this.isIncomplete) {
      const editorContainer = document.createElement("div");
      Object.assign(editorContainer.style, codeBlockContentStyles);
      editorContainer.className = "cm-ai-codeblock-content";
      const languageSupport = this.language ? getLanguageSupport(this.language) : null;
      const extensions = [
        EditorView5.editable.of(false),
        EditorView5.lineWrapping,
        EditorState.readOnly.of(true),
        oneDark,
        EditorView5.theme({
          "&": {
            backgroundColor: "transparent !important"
          },
          ".cm-content": {
            padding: "0 !important",
            fontFamily: "var(--cm-font-family-mono, ui-monospace, SFMono-Regular, SF Mono, Menlo, Consolas, Liberation Mono, monospace) !important",
            fontSize: "13px !important",
            lineHeight: "1.5 !important"
          },
          ".cm-line": {
            padding: "0 !important"
          },
          ".cm-scroller": {
            height: "auto !important",
            flex: "initial !important",
            width: "100% !important"
          }
        })
      ];
      if (languageSupport) {
        extensions.push(languageSupport);
      }
      this.view = new EditorView5({
        state: EditorState.create({
          doc: this.code,
          extensions
        }),
        parent: editorContainer
      });
      wrapper.appendChild(editorContainer);
    }
    return wrapper;
  }
}
var addCodeBlock = StateEffect4.define();
var removeCodeBlocks = StateEffect4.define();
var codeBlockState = StateField4.define({
  create() {
    return Decoration.none;
  },
  update(decorations, tr) {
    decorations = decorations.map(tr.changes);
    for (const effect of tr.effects) {
      if (effect.is(addCodeBlock)) {
        const { from, to, code, language } = effect.value;
        const widget = new CodeBlockWidget(code, language);
        const deco = Decoration.replace({
          widget,
          inclusive: true,
          block: true
        });
        decorations = decorations.update({
          add: [{ from, to, value: deco }],
          filter: (from2, to2) => from2 >= effect.value.to || to2 <= effect.value.from
        });
      } else if (effect.is(removeCodeBlocks)) {
        decorations = Decoration.none;
      }
    }
    return decorations;
  },
  provide: (f) => EditorView5.decorations.from(f)
});
var parseCodeBlocks = (text) => {
  const codeBlocks = [];
  const codeBlockRegex = /```([a-zA-Z]*)\n([\s\S]*?)\n```/g;
  let match;
  while ((match = codeBlockRegex.exec(text)) !== null) {
    codeBlocks.push({
      from: match.index,
      to: match.index + match[0].length,
      language: match[1] || null,
      code: match[2]
    });
  }
  return codeBlocks;
};

// src/ai/ai.ts
import { streamText } from "ai";
import { createOpenAI } from "@ai-sdk/openai";
import { createMistral } from "@ai-sdk/mistral";
import { createGoogleGenerativeAI } from "@ai-sdk/google";
var DEFAULT_MODEL = "openai:gpt-4o";
var debug4 = (...args) => logger_default.debug({ module: "AI" }, args.join(" "));
var createAIService = () => {
  const generateTextFn = async ({
    modelName,
    prompt,
    editorContent,
    apiKey,
    onTextContent
  }) => {
    if (!apiKey) {
      throw new Error("API key is required. Please configure it in settings.");
    }
    const fullPrompt = `Current Editor Content:
${editorContent}

${prompt}`;
    let result;
    let fullText = "";
    debug4("Generating text for model:", modelName);
    debug4("Prompt:", fullPrompt);
    try {
      const systemPrompt = "You are a helpful assistant that can help with coding tasks.";
      switch (modelName) {
        case "openai:gpt-4o": {
          const simulatedResponse = [
            `# Here's a simulated response with code examples

`,
            "## Let's start with a simple inline code example: ",
            "`console.log('hello world')`\n\n",
            "Here's a multiline code block:\n\n```jav",
            `ascript
function greet(name) {
`,
            "    console.log(`Hello, ${name}!`)\n    return {\n}\n",
            "    console.log(`Hello, ${name}!`)\n    return {\n",
            `        message: 'Greeting sent',
        timestamp: new Date()
    }
}

`,
            `// Example usage
const result = greet('Developer')
\`\`\`

`,
            "You can also use inline code for variables like `result` or ",
            `\`name\`.

Here's another code block with a different language:

`,
            "```python\ndef calculate_sum(numbers):\n    total = 0\n",
            `    for num in numbers:
        total += num
    return total

`,
            '# Test the function\nnumbers = [1, 2, 3, 4, 5]\nprint(f"Sum: {calculate_sum(numbers)}")\n```'
          ];
          result = {
            textStream: async function* () {
              for (const chunk of simulatedResponse) {
                await new Promise((resolve) => setTimeout(resolve, 400));
                yield chunk;
              }
            }()
          };
          break;
        }
        case "mistral:large": {
          const mistralClient = createMistral({
            apiKey
          });
          result = streamText({
            model: mistralClient("mistral-large-latest"),
            system: systemPrompt,
            prompt: fullPrompt
          });
          break;
        }
        case "google:gemini-2.0-flash-001": {
          const googleClient = createGoogleGenerativeAI({
            apiKey
          });
          result = streamText({
            model: googleClient("gemini-2.0-flash-001"),
            system: systemPrompt,
            prompt: fullPrompt
          });
          break;
        }
        default: {
          console.warn(`Unknown model: ${modelName}. Using default model.`);
          const openaiClient = createOpenAI({
            apiKey,
            compatibility: "strict"
          });
          result = streamText({
            model: openaiClient("gpt-4"),
            system: systemPrompt,
            prompt: fullPrompt
          });
        }
      }
      for await (const textPart of result.textStream) {
        fullText += textPart;
        onTextContent?.(fullText);
      }
      return fullText;
    } catch (error) {
      debug4("Error:", error);
      throw error;
    }
  };
  return {
    generateText: generateTextFn
  };
};
var aiService = createAIService();

// src/assistant/constants.ts
var AVAILABLE_MODELS = [
  {
    id: "gemini-pro",
    provider: "google",
    name: "Gemini Pro",
    description: "Google's most capable model for text generation"
  },
  {
    id: "gpt-4",
    provider: "openai",
    name: "GPT-4o",
    description: "OpenAI's most capable model"
  },
  {
    id: "mistral-large",
    provider: "mistral",
    name: "Mistral Large",
    description: "Mistral's largest open model"
  }
];

// src/assistant/lib/utils.ts
import { javascript as javascript3 } from "@codemirror/lang-javascript";
import { markdown as markdown3 } from "@codemirror/lang-markdown";
import { python as python3 } from "@codemirror/lang-python";
var mapModelToAIService = (modelId) => {
  switch (modelId) {
    case "gemini-pro":
      return "google:gemini-2.0-flash-001";
    case "gpt-4":
      return "openai:gpt-4o";
    case "mistral-large":
      return "mistral:large";
    default:
      return DEFAULT_MODEL;
  }
};
var formatProviderName = (provider) => {
  switch (provider) {
    case "openai":
      return "OpenAI";
    default:
      return provider.charAt(0).toUpperCase() + provider.slice(1);
  }
};

// src/assistant/lib/renderer.ts
var debug5 = (...args) => console.log("[Assistant Renderer]", ...args);
function renderAssistantPanel(dom, view) {
  const state = view.state.field(assistantState);
  if (!document.querySelector("#cm-spin-keyframes")) {
    const style = document.createElement("style");
    style.id = "cm-spin-keyframes";
    style.textContent = `
            @keyframes cm-spin {
                0% { transform: rotate(0deg); }
                100% { transform: rotate(360deg); }
            }
        `;
    document.head.appendChild(style);
  }
  if (state.showSettings) {
    renderSettingsPanel(dom, view);
    return;
  }
  dom.innerHTML = "";
  Object.assign(dom.style, containerStyles);
  const tabsContainer = crelt3("div");
  Object.assign(tabsContainer.style, tabsContainerStyles);
  const tabsGroup = crelt3("div");
  Object.assign(tabsGroup.style, tabsGroupStyles);
  const createTab = (label, id) => {
    const tab = crelt3("button", {}, label);
    Object.assign(tab.style, getTabStyles(state.activeTab === id));
    tab.onclick = () => view.dispatch({ effects: switchTabEffect.of(id) });
    return tab;
  };
  tabsGroup.appendChild(createTab("Assistant", "assistant"));
  tabsGroup.appendChild(createTab("Agent", "agent"));
  const controlsContainer = crelt3("div");
  Object.assign(controlsContainer.style, controlsContainerStyles);
  const modelSelect = crelt3("select");
  Object.assign(modelSelect.style, modelSelectStyles);
  AVAILABLE_MODELS.forEach((model) => {
    const option = crelt3("option", { value: model.id });
    option.textContent = model.name;
    option.style.direction = "ltr";
    option.style.textAlign = "right";
    if (model.id === state.selectedModel) {
      option.selected = true;
    }
    modelSelect.appendChild(option);
  });
  modelSelect.addEventListener("change", () => {
    view.dispatch({
      effects: selectModelEffect.of(modelSelect.value)
    });
  });
  const settingsButton = crelt3("button", {}, "\u2699\uFE0F");
  Object.assign(settingsButton.style, settingsButtonStyles);
  settingsButton.addEventListener("mouseover", () => {
    settingsButton.style.opacity = "1";
  });
  settingsButton.addEventListener("mouseout", () => {
    settingsButton.style.opacity = "0.7";
  });
  settingsButton.addEventListener("click", () => {
    view.dispatch({
      effects: toggleSettingsEffect.of(true)
    });
  });
  controlsContainer.appendChild(modelSelect);
  controlsContainer.appendChild(settingsButton);
  tabsContainer.appendChild(tabsGroup);
  tabsContainer.appendChild(controlsContainer);
  dom.appendChild(tabsContainer);
  const headerBorder = crelt3("div");
  Object.assign(headerBorder.style, headerBorderStyles);
  dom.appendChild(headerBorder);
  const messagesContainer = crelt3("div");
  Object.assign(messagesContainer.style, messagesContainerStyles);
  const scrollToBottom = () => {
    requestAnimationFrame(() => {
      messagesContainer.scrollTop = messagesContainer.scrollHeight;
    });
  };
  state.messages.forEach((message) => {
    renderMessage(message, messagesContainer, scrollToBottom);
  });
  dom.appendChild(messagesContainer);
  const inputContainer = crelt3("div");
  Object.assign(inputContainer.style, inputContainerStyles);
  const textarea = crelt3("textarea", {
    placeholder: "Ask anything, @ to mention, \u2191 to select",
    rows: "3",
    onkeydown: async (e) => {
      if (e.key === "Enter" && !e.shiftKey) {
        e.preventDefault();
        const target = e.target;
        const content = target.value.trim();
        if (content) {
          debug5("Sending message:", content);
          const selectedModel = AVAILABLE_MODELS.find((m) => m.id === state.selectedModel);
          if (!selectedModel) {
            console.error("Selected model not found");
            return;
          }
          const apiKey = state.apiKeys[selectedModel.provider];
          if (!apiKey) {
            view.dispatch({
              effects: addMessageEffect.of({
                id: crypto.randomUUID(),
                role: "assistant",
                content: `Error: Please configure the API key for ${selectedModel.name} in settings.`,
                status: "complete"
              })
            });
            return;
          }
          target.value = "";
          const userMessage = {
            id: crypto.randomUUID(),
            role: "user",
            content,
            status: "complete"
          };
          view.dispatch({
            effects: addMessageEffect.of(userMessage)
          });
          const assistantMessage = {
            id: crypto.randomUUID(),
            role: "assistant",
            content: "",
            status: "streaming"
          };
          view.dispatch({
            effects: addMessageEffect.of(assistantMessage)
          });
          try {
            const editorContent = view.state.doc.toString();
            let streamedContent = "";
            const aiResponse = await aiService.generateText({
              modelName: mapModelToAIService(state.selectedModel),
              prompt: content,
              editorContent,
              apiKey,
              onTextContent: (text) => {
                streamedContent = text;
                view.dispatch({
                  effects: [
                    updateMessageStatusEffect.of({
                      message: assistantMessage,
                      status: "streaming",
                      content: text
                    })
                  ]
                });
                const messagesContainer2 = view.dom.querySelector(".cm-assistant-content")?.querySelector('div[style*="overflow-y: auto"]');
                if (messagesContainer2) {
                  messagesContainer2.scrollTop = messagesContainer2.scrollHeight;
                }
              }
            });
            view.dispatch({
              effects: [
                updateMessageStatusEffect.of({
                  message: assistantMessage,
                  status: "complete",
                  content: streamedContent || aiResponse
                })
              ]
            });
          } catch (error) {
            view.dispatch({
              effects: [
                updateMessageStatusEffect.of({
                  message: assistantMessage,
                  status: "complete",
                  content: `Error: ${error instanceof Error ? error.message : "An unknown error occurred"}`
                })
              ]
            });
          }
          requestIdleCallback(() => {
            const textareas = view.dom.getElementsByTagName("textarea");
            if (textareas.length > 0) {
              textareas[0].focus();
            }
          });
        }
      }
    }
  });
  Object.assign(textarea.style, textareaStyles);
  textarea.addEventListener("focus", () => {
    textarea.style.borderColor = "var(--cm-border-color, rgba(255, 255, 255, 0.3))";
  });
  textarea.addEventListener("blur", () => {
    textarea.style.borderColor = "var(--cm-border-color, rgba(255, 255, 255, 0.1))";
  });
  inputContainer.appendChild(textarea);
  dom.appendChild(inputContainer);
}
function renderSettingsPanel(dom, view) {
  const state = view.state.field(assistantState);
  dom.innerHTML = "";
  Object.assign(dom.style, settingsPanelContainerStyles);
  const headerContainer = crelt3("div");
  Object.assign(headerContainer.style, tabsContainerStyles);
  const header = crelt3("div", {}, "API Key Settings");
  Object.assign(header.style, settingsHeaderStyles);
  const backButton = crelt3("button", {}, "\u2190");
  Object.assign(backButton.style, settingsButtonStyles);
  backButton.addEventListener("mouseover", () => {
    backButton.style.opacity = "1";
  });
  backButton.addEventListener("mouseout", () => {
    backButton.style.opacity = "0.7";
  });
  backButton.addEventListener("click", () => {
    view.dispatch({
      effects: toggleSettingsEffect.of(false)
    });
  });
  headerContainer.appendChild(header);
  headerContainer.appendChild(backButton);
  dom.appendChild(headerContainer);
  const headerBorder = crelt3("div");
  Object.assign(headerBorder.style, headerBorderStyles);
  dom.appendChild(headerBorder);
  const description = crelt3("div", {}, "Configure API keys for each model provider. Keys are stored locally and never transmitted.");
  Object.assign(description.style, settingsDescriptionStyles);
  dom.appendChild(description);
  const inputsContainer = crelt3("div");
  Object.assign(inputsContainer.style, settingsInputsContainerStyles);
  AVAILABLE_MODELS.forEach((model) => {
    const container = crelt3("div");
    Object.assign(container.style, {
      display: "flex",
      flexDirection: "column",
      gap: "2px"
    });
    const label = crelt3("label", {}, formatProviderName(model.provider));
    Object.assign(label.style, providerHeaderStyles);
    const contentContainer = crelt3("div");
    Object.assign(contentContainer.style, providerContentStyles);
    const resetButton = crelt3("button", {}, "Reset key");
    Object.assign(resetButton.style, {
      ...settingsButtonStyles,
      display: state.apiKeys[model.provider] ? "block" : "none",
      whiteSpace: "nowrap",
      minWidth: "fit-content"
    });
    resetButton.addEventListener("mouseover", () => {
      resetButton.style.opacity = "1";
    });
    resetButton.addEventListener("mouseout", () => {
      resetButton.style.opacity = "0.7";
    });
    resetButton.addEventListener("click", () => {
      input.value = "";
      view.dispatch({
        effects: setApiKeyEffect.of({
          provider: model.provider,
          key: ""
        })
      });
    });
    label.appendChild(resetButton);
    const messageContainer = crelt3("div");
    Object.assign(messageContainer.style, settingsMessageContainerStyles(!!state.apiKeys[model.provider]));
    if (state.apiKeys[model.provider]) {
      const checkmark = crelt3("span", {}, "\u2713");
      Object.assign(checkmark.style, {
        color: "var(--cm-success-color, #4caf50)",
        marginRight: "4px"
      });
      messageContainer.appendChild(checkmark);
      messageContainer.appendChild(document.createTextNode("API key configured."));
    }
    const inputGroup = crelt3("div");
    Object.assign(inputGroup.style, getSettingsInputStyles(!!state.apiKeys[model.provider]));
    const input = crelt3("input", {
      type: "password",
      value: state.apiKeys[model.provider] || "",
      placeholder: model.provider === "google" ? "AIzaSy..." : model.provider === "openai" ? "sk-000000000000000000000000000000000000000000000000" : "ottggm..."
    });
    Object.assign(input.style, settingsInputFieldStyles);
    input.addEventListener("keydown", (e) => {
      if (e.key === "Enter") {
        input.blur();
      }
    });
    input.addEventListener("blur", () => {
      if (input.value) {
        view.dispatch({
          effects: setApiKeyEffect.of({
            provider: model.provider,
            key: input.value
          })
        });
      }
    });
    input.addEventListener("focus", () => {
      input.style.borderColor = "var(--cm-accent-color, #4a9eff)";
      input.style.boxShadow = "0 0 0 1px var(--cm-accent-color, #4a9eff)";
    });
    input.addEventListener("blur", () => {
      input.style.borderColor = "var(--cm-border-color, rgba(255, 255, 255, 0.1))";
      input.style.boxShadow = "none";
    });
    inputGroup.appendChild(input);
    contentContainer.appendChild(messageContainer);
    contentContainer.appendChild(inputGroup);
    container.appendChild(label);
    container.appendChild(contentContainer);
    inputsContainer.appendChild(container);
  });
  dom.appendChild(inputsContainer);
}
function renderMessage(message, container, scrollToBottom) {
  if (message.status === "streaming" && !message.content) {
    const loadingContainer = crelt3("div");
    Object.assign(loadingContainer.style, loadingContainerStyles);
    const spinner = crelt3("div");
    Object.assign(spinner.style, spinnerStyles);
    loadingContainer.appendChild(spinner);
    container.appendChild(loadingContainer);
  }
  if (message.content || message.status !== "streaming") {
    const messageEl = crelt3("div");
    Object.assign(messageEl.style, getMessageStyles(message.role === "user"));
    const contentEl = crelt3("div");
    Object.assign(contentEl.style, messageContentStyles);
    const codeBlocks = parseCodeBlocks(message.content);
    let lastPos = 0;
    const segments = [];
    codeBlocks.forEach((block) => {
      if (block.from > lastPos) {
        segments.push({
          type: "text",
          content: message.content.slice(lastPos, block.from)
        });
      }
      segments.push({
        type: "code",
        content: block.code,
        language: block.language
      });
      lastPos = block.to;
    });
    if (message.status === "streaming") {
      const remainingContent = message.content.slice(lastPos);
      const incompleteBlockMatch = /```([a-zA-Z]*)\n([\s\S]*)$/.exec(remainingContent);
      if (incompleteBlockMatch) {
        const incompleteBlockStart = remainingContent.lastIndexOf("```");
        if (incompleteBlockStart > 0) {
          segments.push({
            type: "text",
            content: remainingContent.slice(0, incompleteBlockStart)
          });
        }
        segments.push({
          type: "incomplete-code",
          content: "",
          language: incompleteBlockMatch[1] || null
        });
      } else {
        if (remainingContent) {
          segments.push({
            type: "text",
            content: remainingContent
          });
        }
      }
    } else {
      if (lastPos < message.content.length) {
        segments.push({
          type: "text",
          content: message.content.slice(lastPos)
        });
      }
    }
    segments.forEach((segment) => {
      if (segment.type === "code" || segment.type === "incomplete-code") {
        const codeBlockContainer = crelt3("div");
        Object.assign(codeBlockContainer.style, codeBlockContainerStyles);
        const widget = new CodeBlockWidget(segment.type === "code" ? segment.content : "", segment.language || null, segment.type === "incomplete-code");
        codeBlockContainer.appendChild(widget.toDOM());
        contentEl.appendChild(codeBlockContainer);
      } else if (segment.type === "text") {
        const textContainer = crelt3("div");
        Object.assign(textContainer.style, textContainerStyles);
        const inlineCodeRegex = /(?<!`)`([^`]+)`(?!`)/g;
        const processedText = segment.content.replace(inlineCodeRegex, (_, code) => `<code style="${inlineCodeStyles}">${code}</code>`);
        textContainer.innerHTML = processedText;
        contentEl.appendChild(textContainer);
      }
    });
    messageEl.appendChild(contentEl);
    container.appendChild(messageEl);
    scrollToBottom();
  }
}

// src/assistant/index.ts
import { Facet as Facet2 } from "@codemirror/state";
var debug6 = (...args) => logger_default.debug("[Assistant]", ...args);
function setupPanelKeyboardShortcuts(dom, view, keyConfig) {
  const isMac = /Mac/.test(navigator.platform);
  const keys = typeof keyConfig === "string" ? { mac: keyConfig, win: keyConfig } : keyConfig;
  const targetKey = (isMac ? keys.mac : keys.win)?.toLowerCase();
  if (!targetKey)
    return;
  debug6("Setting up panel keyboard shortcuts:", { isMac, targetKey });
  dom.addEventListener("keydown", (event) => {
    const modKey = isMac ? event.metaKey : event.ctrlKey;
    if (!modKey)
      return;
    const pressedKey = `${isMac ? "cmd" : "ctrl"}-${event.key.toLowerCase()}`;
    debug6("Key pressed in panel:", pressedKey);
    if (pressedKey === targetKey) {
      event.preventDefault();
      event.stopPropagation();
      debug6("Executing panel keyboard shortcut:", targetKey);
      toggleSidebarCommand(view, "ai-assistant");
    }
  });
}
var assistantPanelSpec = {
  id: "ai-assistant",
  create(view) {
    const dom = document.createElement("div");
    dom.className = "cm-assistant-content";
    renderAssistantPanel(dom, view);
    const keymapOpt = view.state.facet(assistantKeymap);
    if (keymapOpt) {
      setupPanelKeyboardShortcuts(dom, view, keymapOpt);
    }
    requestIdleCallback(() => {
      const textareas = dom.getElementsByTagName("textarea");
      if (textareas.length > 0) {
        textareas[0].focus();
      }
    });
    return dom;
  },
  update(view) {
    const dom = view.dom.querySelector(".cm-assistant-content");
    if (dom) {
      renderAssistantPanel(dom, view);
    }
  }
};
var assistantKeymap = Facet2.define({
  combine: (values) => values[0]
});
var assistantPlugin = ViewPlugin4.fromClass(class {
  update(update) {
    if (update.state.field(assistantState) !== update.startState.field(assistantState)) {
      const dom = update.view.dom.querySelector(".cm-assistant-content");
      if (dom) {
        renderAssistantPanel(dom, update.view);
      }
    }
  }
});
function assistant(options = {}) {
  const {
    keymap: keymapOpt,
    initiallyOpen = false,
    ...sidebarOptions
  } = options;
  return [
    ...createSidebar({
      ...sidebarOptions,
      id: "ai-assistant",
      dock: "right",
      overlay: true,
      initiallyOpen,
      initialPanelId: "ai-assistant"
    }),
    assistantState,
    sidebarPanel.of(assistantPanelSpec),
    assistantPlugin,
    keymapOpt ? assistantKeymap.of(keymapOpt) : [],
    createSidebarKeymap("ai-assistant", keymapOpt)
  ];
}
// src/hooks/index.ts
import { useEffect, useRef } from "react";
import { EditorState as EditorState2 } from "@codemirror/state";
import { EditorView as EditorView6 } from "@codemirror/view";
import { javascript as javascript4 } from "@codemirror/lang-javascript";
import { oneDark as oneDark2 } from "@codemirror/theme-one-dark";
import { basicSetup } from "codemirror";
function useEditor(config = {}) {
  const editorRef = useRef(null);
  const viewRef = useRef(null);
  useEffect(() => {
    if (!editorRef.current)
      return;
    const {
      initialContent = "",
      explorer: explorerConfig,
      assistant: assistantConfig,
      extensions = []
    } = config;
    const state = EditorState2.create({
      doc: initialContent,
      extensions: [
        basicSetup,
        javascript4(),
        oneDark2,
        ...explorerConfig ? [
          explorer({
            dock: "left",
            width: "250px",
            overlay: false,
            backgroundColor: "#2c313a",
            initiallyOpen: false,
            ...explorerConfig
          })
        ] : [],
        ...assistantConfig ? [
          assistant({
            width: "400px",
            backgroundColor: "#2c313a",
            initiallyOpen: false,
            ...assistantConfig
          })
        ] : [],
        ...extensions,
        EditorView6.theme({
          "&": {
            height: "100vh",
            display: "flex !important",
            flexDirection: "row !important"
          },
          ".cm-scroller": {
            overflow: "auto",
            flex: "1 1 auto !important",
            minWidth: "0 !important"
          },
          ".cm-content": {
            minHeight: "100%"
          }
        })
      ]
    });
    const view = new EditorView6({
      state,
      parent: editorRef.current
    });
    if (explorerConfig?.initialFiles) {
      view.dispatch({
        effects: updateFilesEffect.of(explorerConfig.initialFiles)
      });
    }
    viewRef.current = view;
    return () => {
      view.destroy();
      viewRef.current = null;
    };
  }, [config]);
  return {
    ref: editorRef,
    view: viewRef.current
  };
}
export {
  useEditor
};
